//*!!Code automatically generated by 'ROBOTC' configuration wizard               !!*//

// Receiver.c
//
// Receive multiple data types and display the values received on a scrolling
// display on the NXT screen. Also log verbosely to the debug consle in RobotC
//
// The receiver listens to the NXTBee, polling every 0.5 seconds for a message.
// If a regular data value message is received the value is printed on the NXT
// screen. If a packed message format is received the message is iteratively
// parsed by calling getNextType repeatedly to extract each data type from
// the packed format.
//
// Run this program on the NXT with the NXTBee on port 4. It assumes that the
// NXTBee is configured to the correct baud rate in NXTBeeComms.h
//
// Run the Packer.pde sketch in Processing to send packed data to the NXT
// using an XStick connected to your Mac/PC.
//
// Copyright 2012 Mark Crosbie mark@mastincrosbie.com
// http://www.mastincrosbie.com/Marks_LEGO_projects
//


#include "NXTBeeComms.h"

// default receive messages from NXT 1, I am NXT 2
#define OTHER_NXT 1
#define MY_ADDRESS 2

//////////////////////////////////////////////////////////////////////
// parsePackedMsg
//
// Parse a packed data message as an example of how to extract values
// Prints the data values on the scrolling NXT display
// If a packed type is encountered inside a packed type then return
//
//////////////////////////////////////////////////////////////////////
void parsePackedMsg(int currPtr) {

  byte msgType;
  int ivalue;
  long lvalue;
  byte bvalue;
  bool boolValue;
  string s;
  byte b[100];

  while( (msgType = getNextType(currPtr)) != -1) {

      //writeDebugStreamLine("msgType = %d", msgType);

	    // depending on the data type in the message extract the value
	    // transmitted
	    if(msgType == INT_TYPE) {
	      ivalue = unpackValue(currPtr);
	      nxtScrollText("Int: %d", ivalue);
	      writeDebugStreamLine("Int = %d", ivalue);
	    }

	    if(msgType == BYTE_TYPE) {
	      bvalue = unpackValue(currPtr);
	      nxtScrollText("Byte: %d", bvalue);
	      writeDebugStreamLine("Byte = %d", bvalue);
	    }

	    if(msgType == LONG_TYPE) {
	      lvalue = unpackValue(currPtr);
	      nxtScrollText("Long: %d", lvalue);
	      writeDebugStreamLine("Long = %d", lvalue);
	    }

	    if(msgType == ASCII_STRING_TYPE) {
	      unpackString(currPtr, s);
	      nxtScrollText(s);
	      writeDebugStreamLine(s);
	    }

	    if(msgType == BYTE_STRING_TYPE) {
	      int byteCount = unpackBytes(currPtr, b);
	      nxtScrollText("%d bytes", byteCount);
	      for(int i=0; i < byteCount; i++)
	        writeDebugStream("%d ", b[i]);
	      writeDebugStreamLine("");
	    }

	    if(msgType == BOOL_TYPE) {
	      lvalue = unpackValue(currPtr);
	      writeDebugStreamLine("Boolean: %d", lvalue);
	      if(lvalue)
	        nxtScrollText("Bool: true");
	      else
	        nxtScrollText("Bool: false");
	    }

	    if(msgType == PACKED_TYPE) {
	      return;
	    }
  }
}

task main() {

  int ivalue, len;
  long lvalue;
  byte bvalue;
  bool boolValue;
  string s;
  byte b[10];

  msgHdr_t h;

  eraseDisplay();
  bNxtLCDStatusDisplay = false;
  setupHighSpeedLink();

  nxtDisplayString(1, "NXTBee Recv");

  setMyAddress(MY_ADDRESS);

  // loop forever reading messages until orange button
  // is pressed
  while(nNxtButtonPressed != kEnterButton) {

    // did we receive a valid message?
    if( (len = Receive(h)) > 0) {

    // depending on the data type in the message extract the value
	    // transmitted
	    if(h.msgType == INT_TYPE) {
	      ivalue = getValue();
	      nxtScrollText("Int: %d", ivalue);
	      //writeDebugStreamLine("Value = %d", ivalue);
	    }

	    if(h.msgType == BYTE_TYPE) {
	      bvalue = getValue();
	      nxtScrollText("Byte: %d", bvalue);
	      //writeDebugStreamLine("Value = %d", value);
	    }

	    if(h.msgType == LONG_TYPE) {
	      lvalue = getValue();
	      nxtScrollText("Long %d", lvalue);
	      //writeDebugStreamLine("Value = %d", value);
	    }

	    if(h.msgType == ASCII_STRING_TYPE) {
	      getString(s);
	      nxtScrollText(s);
	    }

	    if(h.msgType == BYTE_STRING_TYPE) {
	      int byteCount = getBytes(b);
	      nxtScrollText("%d bytes", byteCount);
	    }

	    if(h.msgType == PACKED_TYPE) {
	      nxtScrollText("Packed Msg");
	      writeDebugStreamLine("Packed Message");
	      parsePackedMsg(START_OF_MSG);
	    }
	}

	if(len == ECKSUM) {
	  nxtScrollText("Bad checksum");
	}

    wait10Msec(50);
  }
}
